var fontFamily = '"Trebuchet MS", Verdana, Helvetica, Sans-Serif';
var languageCode = null;

try {
    getBrowserLanguageCode();
    if(languageCode == null)
    {
           if (navigator.userLanguage) // Internet Explorer
                  languageCode = navigator.userLanguage;
           else if (navigator.language) // Firefox
                  languageCode = navigator.language;
           else languageCode = 'en-us';
    }
}
catch(e) {
	languageCode = 'en-us';
}
switch(languageCode.toLowerCase()) {
	case "zh-cn":
	case "zh":
		fontFamily = '"Microsoft YaHei", "Hiragino Sans GB", "Trebuchet MS", "Arial Unicode", sans-serif';
		break;
	case "ja-jp":
	case "ja":
		fontFamily = '"Meiryo UI", "MS UI Gothic", "Hiragino Kaku Gothic", "Trebuchet MS", "Arial Unicode", sans-serif';
		break;
	case "ko-kr":
	case "ko":
		fontFamily = '"Malgun Gothic", "맑은 고딕", "Gulim", "Trebuchet MS", "Arial Unicode", "sans-serif"';
		break;
}
document.observe("dom:loaded", function() {
	document.getElementsByTagName("body")[0].style.fontFamily = fontFamily;
	
	var head = document.getElementsByTagName('head')[0],
    style = document.createElement('style'),
    rules = document.createTextNode('input, textarea { font-family: ' + fontFamily + '; }');

	style.type = 'text/css';
	if(style.styleSheet)
	    style.styleSheet.cssText = rules.nodeValue;
	else style.appendChild(rules);
	head.appendChild(style);

});

//get the browser language code
function getBrowserLanguageCode() {
           new Ajax.Request(urlConfig.getClientLocale, {
              method: 'get',
              asynchronous: false,
              onComplete: function(transport) {
                     var    data = transport.responseText.evalJSON(true);
                     if (data.getClientLocaleResponse) {
                        languageCode = data.getClientLocaleResponse;
                     }
              }
       });
}


// utility function - precaches images for quick loading time of initially hidden elements
function preCacheImage(imageUrl) {

	var img = new Image();
	img.src = imageUrl;

}

// main background
preCacheImage('images/body/bd_main.2000x1400.jpg');


// edit mode
preCacheImage('images/containers/ct_editTopBtm.png');
preCacheImage('images/containers/ct_editCenter.png');

preCacheImage('images/tables/ta_sideWhiteEditMode.png');
preCacheImage('images/tables/ta_tableRightEditMode.png');


// tooltips
preCacheImage('images/navigation/nv_toolTip.png');
preCacheImage('images/messages/ms_toolTip.png');


// right arrows
preCacheImage('images/misc/ms_rtArBlue.png');
preCacheImage('images/misc/ms_rtArOrange.png');


// drop down opens
preCacheImage('images/controls/co_ddOpen.png');
preCacheImage('images/controls/co_ddOpenTop.png');
preCacheImage('images/controls/co_ddOpenLeft.png');


// table backgrounds
preCacheImage('images/tables/ta_cellWhite.png');
preCacheImage('images/tables/ta_cellGrey.png');
preCacheImage('images/tables/ta_cellBlue.png');
preCacheImage('images/tables/ta_cellOrange.png');

preCacheImage('images/tables/ta_sideWhite.png');
preCacheImage('images/tables/ta_sideGrey.png');
preCacheImage('images/tables/ta_sideBlue.png');
preCacheImage('images/tables/ta_sideOrange.png');



var Translator = {
	translations: {},
	initialize: function () {
		var request = new Ajax.Request('localization/client_translations.js', {
			asynchronous: false,
			method: 'get',
			onSuccess: function (transport) {
				var propertiesFile = transport.responseText;
				var lines = propertiesFile.split("\n");

				for (var i = 0; i < lines.length; i++) {
					var line = lines[i];
					line = line.replace("\r", "");
					if (line == "")
						continue;

					var lineSplit = line.split("=");
					if (lineSplit.length != 2) {
						alert("translations file has invalid line# " + (i + 1));
						return;
					}
					var key = lineSplit[0];
					var value = lineSplit[1];

					if (key == null || key == "") {
						alert("empty key provided in translations file at line# " + (i + 1));
						return;
					}

					if (Translator.translations[key]) {
						alert("duplicate translation entry -- trying to replace value for key '" + key + "' at line# " + (i + 1));
						return;
					}

					Translator.translations[key] = value == null ? "" : value;
				}
			},
			onFailure: function () {
				alert("couldn't get client translations!!");
			}
		});
	},

	translate: function (translationName, replaceTokenValuesObject) {
		if (!Translator.translations[translationName]) {
			return "TEXTNOTFOUND_" + translationName;
		}

		var debug = false;

		var retVal = Translator.translations[translationName];

		if (replaceTokenValuesObject) {
			for (var prop in replaceTokenValuesObject) {
				retVal = retVal.replace(new RegExp('%%' + prop + '%%', 'g'), replaceTokenValuesObject[prop]);
			};
		}

		if (debug) {
			return "!!CLIENT_LOCALIZED!!";
			//return "!!" + retVal + "!!";
		}

		return retVal;
	}
};
Translator.initialize();

// utility function - trims 'px' from style value, and returns integer value
function getValueFromStyle(str) {

	return parseInt(str.substring(0, (str.length - 2)));
}

// special section task values - initially cleared
var specialSectionTaskType = null;
var specialSectionTaskUrl = null;

// animation speed values
var containerMinMaxSpeed = 0.2;
var msgBoxCloseSpeed = 0.2;
var paginationScrollSpeed = 0.4;
var drawerOpenCloseSpeed = 0.2;

// toggles container min/max state
function toggleContainerMinMax(containerId) {

	if ($(containerId)) {
		// check to see if container has min/max button, and does not have disabled class set
		if ($(containerId).down('.minMaxButton') && !$(containerId).down('.minMaxButton').hasClassName('disabled')) {

			if ($(containerId).hasClassName('minimized')) {
				// first, toggle minimized class for container
				$(containerId).toggleClassName('minimized');
				// blind down all section centers
				$(containerId).select('.sectionContainer').each(function (item) { item.blindDown({ duration: containerMinMaxSpeed }); });

			} else {
				// blind up all section centers
				$(containerId).select('.sectionContainer').each(function (item) { item.blindUp({ duration: containerMinMaxSpeed }); });
				// after effect finishes, toggle minimized class for container  **use delay so that it happens after the blind ups are finished
				window.setTimeout(function () {
					$(containerId).toggleClassName('minimized');
				}, containerMinMaxSpeed * 1000);
			}


		}
	}

}

// disable section menu
function disableSectionMenu(sectionId) {

	var sectionMenuId = sectionId + '_menu';

	if ($(sectionMenuId)) {

		// only do it, if not already disabled
		if (!$(sectionMenuId).hasClassName('menuDisabled')) {
			// add section menu disabled class
			$(sectionMenuId).addClassName('menuDisabled');

			// for all contained links, clear href attribute, after backing it up
			$(sectionMenuId).select('a').each(function (item) { item.writeAttribute('href_bak', item.readAttribute('href')); item.writeAttribute('href', false); });
		}
	}
}

// enable section menu
function enableSectionMenu(sectionId) {

	var sectionMenuId = sectionId + '_menu';

	if ($(sectionMenuId)) {

		// only do it, if disabled
		if ($(sectionMenuId).hasClassName('menuDisabled')) {
			// remove section menu disabled class
			$(sectionMenuId).removeClassName('menuDisabled');

			// for all contained links, reset href using backup value, then clear backup value
			$(sectionMenuId).select('a').each(function (item) { item.writeAttribute('href', item.readAttribute('href_bak')); item.writeAttribute('href_bak', false); });
		}
	}
}

// disable table headers
function disableTableHeaders(tableId) {

	if ($(tableId)) {

		// add table headers disabled class
		$(tableId).addClassName('tableHeadersDisabled');

	}
}


// enable table headers
function enableTableHeaders(tableId) {

	if ($(tableId)) {

		// remove table headers disabled class
		$(tableId).removeClassName('tableHeadersDisabled');

	}
}


// clear input field if containing help text
function clearIfHelpText(input, helpText) {
	if (helpText != '' && ($(input).value == helpText)) {
		input.value = '';
		// also, remove help text class
		input.removeClassName('helpText');
	}
	var _el = input;
	if (_el.hasClassName('watermark')) {
		_el.value = '';
		_el.removeClassName('watermark');
		_el.removeClassName('clearOnClick');
	}
	if (_el.hasClassName('watermarkOnEdit')) {
		_el.value = '';
		_el.removeClassName('watermarkOnEdit');
		_el.addClassName('reWatermarkOnEdit');
	}
}


// toggle disabled status of a block of inputs based on another inputs value
function toggleDisableInputBlock(curValue, blockId, disableValue) {

	var disable = false;
	// see if current value is the disabled value
	if (curValue == disableValue) {
		disable = true;
	}

	// find all inputs with the matching 'disabledblockid' value
	$$('input[disableblockid="' + blockId + '"]').each(function (item) {
		if (disable) {
			// clear value
			item.value = '';
			item.checked = false;

			// add disabled class
			item.addClassName('disabled');
			// regular disable call
			item.disable();
			// if in data field row, also disable row by adding disabled class - greys out text
			if (item.up('.dataFieldRow')) {
				item.up('.dataFieldRow').addClassName('disabled');
			}
		} else {
			// remove disabled class
			item.removeClassName('disabled');
			// regular enable call
			item.enable();
			// if in data field row, also enable row by removing disabled class
			if (item.up('.dataFieldRow')) {
				item.up('.dataFieldRow').removeClassName('disabled');
			}
		}
	});
}


// utility function that returns 'attribute=value' string, only if attribute exists
function getAttributeString(item, attribute) {

	if (item.readAttribute(attribute) != null) {
		return attribute + '="' + item.readAttribute(attribute) + '"';
	} else {
		return '';
	}
}


function createAttributeHtmlForvValidation(item) {
	var addValidation = "";

	if (item.readAttribute('data-validate') != null) {
		var vals = item.readAttribute('data-validate');
		addValidation += ' data-validate="' + vals + '"';
	}

	if (item.readAttribute('data-validate-display') != null) {
		var vals = item.readAttribute('data-validate-display');
		addValidation += ' data-validate-display="' + vals + '"';
	}

	return addValidation;
}

// inserts hidden input field
function spanToDefaultInput(item) {

	if (item) {
		var str = '<input type="hidden" class="inputFldDefault" id="' + item.id + '" name="' + item.id + '"'
					+ ' value="' + item.innerHTML.strip() + '" />';

		item.insert(str);
	}

}

// removes hidden input field
function inputToDefaultSpan(item) {

	if (item) {
		item.remove();
	}

}

// replaces span with password input field
function spanToPasswordInput(item) {

	if (item) {
		var value = '';
		var extraClass = '';
		if (item.innerHTML.strip() != '') {
			value = item.innerHTML.strip();
		} else if (item.readAttribute('helptext') != null) {
			// special case if no current value, and help text exists
			value = item.readAttribute('helptext');
			extraClass = ' helpText';
		}

		var validationInfo = createAttributeHtmlForvValidation(item);

		var tabIndex = item.readAttribute('tabindex');
		var str = '<input type="password" ' + validationInfo + ' tabindex="' + tabIndex + '" class="inputFldPassword' + extraClass + '" id="' + item.id + '" name="' + item.id + '"'
					+ ' style="width: ' + Math.ceil(item.readAttribute('size') / 2) + 'em;"' + getAttributeString(item, 'size') + getAttributeString(item, 'helptext')
					+ getAttributeString(item, 'disableblockid')
					+ ' onfocus="clearIfHelpText(this, \'' + item.readAttribute('helptext') + '\'); toggleHighlightRow(\'' + item.id + '\');" onblur="toggleHighlightRow(\'' + item.id + '\');"'
					+ ' value="' + value + '" />';

		item.replace(str);
	}

}

// replaces span with text input field
function spanToTextInput(item) {

	if (item) {
		var value = '';
		var extraClass = '';
		if (item.innerHTML.strip() != '') {
			if (item.readAttribute('title') != null && item.readAttribute('title') != '')
				value = item.readAttribute('title');
			else
			value = item.innerHTML.strip();
		} else if (item.readAttribute('helptext') != null) {
			// special case if no current value, and help text exists
			value = item.readAttribute('helptext');
			extraClass = ' helpText';
		}


		var addValidation = createAttributeHtmlForvValidation(item);

		if (item.hasClassName('watermarkOnEdit') && item.innerHTML == item.readAttribute('helptext')) {
			extraClass += ' watermarkOnEdit';
		}

		var tabIndex = item.readAttribute('tabindex');

		var str = '<input ' + addValidation + ' tabindex="' + tabIndex + '" type="text" class="inputFldText' + extraClass + '" id="' + item.id + '" name="' + item.id + '"'
					+ ' style="width: ' + Math.ceil(item.readAttribute('size') / 2) + 'em;"' + getAttributeString(item, 'size') + getAttributeString(item, 'helptext')
					+ getAttributeString(item, 'disableblockid')
					+ ' onfocus="clearIfHelpText(this, \'' + item.readAttribute('helptext') + '\'); toggleHighlightRow(\'' + item.id + '\');" onblur="toggleHighlightRow(\'' + item.id + '\');"'
					+ ' value="' + value + '" />';

		item.replace(str);
	}

}

// replaces password input field with span
function inputToPasswordSpan(item) {

	if (item) {
		var value = '';
		// make sure current value does not equal help text
		if (item.readAttribute('helptext') == null || item.readAttribute('helptext') != item.getValue) {
			value = item.getValue();
		}
		var addValidation = createAttributeHtmlForvValidation(item);



		var str = '<span id="' + item.id + '" ' + addValidation + ' class="inputFldPassword"' + getAttributeString(item, 'size') + getAttributeString(item, 'helptext') + getAttributeString(item, 'disableblockid') + '>**********</span>';

		item.replace(str);
	}

}

// replaces text input field with span
function inputToTextSpan(item) {

	if (item) {
		var value = '';
		// make sure current value does not equal help text
		if (item.readAttribute('helptext') == null || item.readAttribute('helptext') != item.getValue) {
			value = item.getValue();
		}

		var extraClass = '';
		if (item.hasClassName('reWatermarkOnEdit')) {
			extraClass += ' watermarkOnEdit';
		}

		if (item.value == '' || item.value == item.readAttribute('helptext') || item.value == "domain\\username") {
			value = item.readAttribute('helptext');
			extraClass += ' watermarkOnEdit';
		}

		var title = '';
		if (item.getValue().length > 58) {
			value = item.getValue().substring(0, 54) + '...';
			title = item.getValue();
		}
		var addValidation = createAttributeHtmlForvValidation(item);

		var str = '<span ' + addValidation + ' id="' + item.id + '" class="inputFldText' + extraClass + '"' + getAttributeString(item, 'size') + getAttributeString(item, 'helptext') + getAttributeString(item, 'disableblockid') + ' title="' + title + '">' + value + '</span>';

		item.replace(str);
	}

}

// replaces span with text area field
function spanToTextAreaInput(item) {

	if (item) {
		var value = '';
		var extraClass = '';
		if (item.innerHTML.strip() != '') {
			// replace <br> with line breaks
			value = item.innerHTML.strip().replace('<br>', '\n');
		} else if (item.readAttribute('helptext') != null) {
			// special case if no current value, and help text exists
			value = item.readAttribute('helptext');
			extraClass = ' helpText';
		}

		var addValidation = createAttributeHtmlForvValidation(item);

		var str = '<textarea class="inputFldTextArea' + extraClass + '" ' + addValidation + ' id="' + item.id + '" name="' + item.id + '"'
					+ ' style="width: ' + Math.ceil(item.readAttribute('size') / 2) + 'em;"' + getAttributeString(item, 'size') + getAttributeString(item, 'rows') + getAttributeString(item, 'helptext')
					+ ' onfocus="clearIfHelpText(this, \'' + item.readAttribute('helptext') + '\'); toggleHighlightRow(\'' + item.id + '\');" onblur="toggleHighlightRow(\'' + item.id + '\');"'
					+ '>' + value + '</textarea>';

		item.replace(str);
	}

}

// replaces text area field with span
function inputToTextAreaSpan(item) {

	if (item) {
		var value = '';
		// make sure current value does not equal help text
		if (item.readAttribute('helptext') == null || item.readAttribute('helptext') != item.getValue) {
			// replace line breaks with <br>
			value = item.getValue().replace('\n', '<br>');
		}

		var addValidation = createAttributeHtmlForvValidation(item);

		var str = '<span id="' + item.id + '" ' + addValidation + ' class="inputFldTextArea"' + getAttributeString(item, 'size') + getAttributeString(item, 'rows') + getAttributeString(item, 'helptext') + '>' + value + '</span>';

		item.replace(str);
	}

}

// replaces span with radio buttons container, and radio button input fields
function spanToRadioInput(item) {

	if (item) {
		if (item.readAttribute('optionvals')) {
			var tabIndex = parseInt(item.readAttribute('tabindex'));

			// pull option values by splitting optionvals attribute using '|' delimiter
			var optionVals = item.readAttribute('optionvals').split('|');
			if (optionVals.length > 0) {
				var curVal = item.innerHTML.strip();
				var disabledValueTranslated = Translator.translate(item.readAttribute('disabledvalue'));

				var disabled = false;
				// check to make sure not already disabled by checking current value against disabled default value
				if (item.readAttribute('disabledvalue') != null && disabledValueTranslated == curVal) {
					disabled = true;
				}

				var str = '<span id="' + item.id + '" class="inputFldRadioSpan';
				if (disabled) {
					str = str + ' disabled';
				}
				str = str + '" optionvals="' + item.readAttribute('optionvals') + '"' + getAttributeString(item, 'disabledvalue');
				str = str + getAttributeString(item, 'disableblockid') + getAttributeString(item, 'disableblockcontrollerid') + getAttributeString(item, 'disableblockcontrollervalue') + '>';

				var onchange = '';
				var jsCode = '';
				// special onchange code if this is a disable block controller
				if (item.readAttribute('disableblockcontrollerid') != null && item.readAttribute('disableblockcontrollervalue') != null) {
					onchange = ' onclick="toggleDisableInputBlock(this.value, \'' + item.readAttribute('disableblockcontrollerid') + '\',\'' + item.readAttribute('disableblockcontrollervalue') + '\');"';
					// also, set disabled state based on current value
					jsCode = '<script>toggleDisableInputBlock(\'' + curVal + '\', \'' + item.readAttribute('disableblockcontrollerid') + '\',\'' + item.readAttribute('disableblockcontrollervalue') + '\');</script>';
				}

				var optionArr = new Array();
				// create radio button inputs with label
				for (var i = 0; i < optionVals.length; i++) {
					var translatedOptionVal = Translator.translate(optionVals[i]);
					optionArr[i] = '<input type="radio" class="inputFldRadio" id="' + item.id + '" name="' + item.id + '"' + ' value="' + translatedOptionVal + '"';
					if (translatedOptionVal == curVal) {
						optionArr[i] = optionArr[i] + ' checked="checked"';
					}
					if (disabled) {
						optionArr[i] = optionArr[i] + ' disabled="disabled"';
					}
					optionArr[i] += ' tabindex="' + tabIndex + '"';
					tabIndex++;
					optionArr[i] = optionArr[i] + onchange + getAttributeString(item, 'disableblockid') + ' /><label for="' + item.id + '">' + translatedOptionVal + '</label>';
				}
				str = str + optionArr.join(' ');
				str = str + '</span>';

				str = str + jsCode;

				item.replace(str);
			}
		}
	}

}

// replaces radio buttons container and input fields with span
function inputToRadioSpan(item) {

	if (item) {
		var value = '';
		// determined selected radio
		if (item.hasClassName('disabled') && item.readAttribute('disabledvalue') != null) {
			// use disabled default value
			var translatedDisabledValue = Translator.translate(item.readAttribute('disabledvalue'));
			value = translatedDisabledValue;
		} else {
			// grab first checked item's value
			item.select('input').each(function (input) { if (input.checked) { value = input.getValue(); } });
		}

		var str = '<span id="' + item.id + '" class="inputFldRadio" optionvals="' + item.readAttribute('optionvals') + '"' + getAttributeString(item, 'disabledvalue') + getAttributeString(item, 'disableblockid') + getAttributeString(item, 'disableblockcontrollerid') + getAttributeString(item, 'disableblockcontrollervalue') + '>' + value + '</span>';

		item.replace(str);
	}

}

// replaces span with checkboxes container, and checkbox input fields
function spanToCheckboxInput(item) {

	if (item) {
		if (item.readAttribute('optionvals')) {
			// pull option values by splitting optionvals attribute using '|' delimiter
			var optionVals = item.readAttribute('optionvals').split('|');
			if (optionVals.length > 0) {
				var str = '<span id="' + item.id + '" class="inputFldCheckboxSpan" optionvals="' + item.readAttribute('optionvals') + '">';

				var selectedValsArr = new Array();
				// determine checked items by splitting current inner html with comma and no break special character
				var tmpArr = item.innerHTML.strip().split(',&nbsp;');
				for (var i = 0; i < tmpArr.length; i++) {
					selectedValsArr[tmpArr[i]] = 1;
				}

				var tabIndex = parseInt(item.readAttribute('tabindex'));

				var optionArr = new Array();
				for (var i = 0; i < optionVals.length; i++) {
					var localizedOptionVal = Translator.translate(optionVals[i]);
					optionArr[i] = '<input tabindex="' + tabIndex + '" type="checkbox" class="inputFldCheckbox" id="' + item.id + '" name="' + item.id + '[]"' + ' value="' + localizedOptionVal + '"';
					// see if checked
					if (selectedValsArr[localizedOptionVal]) {
						optionArr[i] = optionArr[i] + ' checked="checked"';
					}
					optionArr[i] = optionArr[i] + ' /><label for="' + item.id + '">' + localizedOptionVal + '</label>';
					tabIndex++;
				}
				str = str + optionArr.join('&nbsp; ');
				str = str + '</span>';

				item.replace(str);
			}
		}
	}

}

// replaces checkboxes container and input fields with span
function inputToCheckboxSpan(item) {

	if (item) {
		var valueArr = new Array();
		// determined selected input items, build array of selected values to be joined by comma and no break special character
		item.select('input').each(function (input) {
			if (input.checked) {
				valueArr[valueArr.length] = input.getValue();
			} else {
				valueArr[valueArr.length] = 'Disabled';
			}
		});

		var str = '<span id="' + item.id + '" class="inputFldCheckbox" optionvals="' + item.readAttribute('optionvals') + '">' + valueArr.join(',&nbsp;') + '</span>';

		item.replace(str);
	}

}


// replaces span with checkboxes container, and checkbox input fields
function spanToDropdownInput(item) {

	if (item) {
		if (item.readAttribute('optionvals')) {
			// pull option values by splitting optionvals attribute using '|' delimiter
			var optionVals = item.readAttribute('optionvals').split('|');
			if (optionVals.length > 0) {
				// build drop down
				var str = '<span id="' + item.id + '" class="inputFldDropdownSpan" optionvals="' + item.readAttribute('optionvals') + '">';

				str = str + '<div id="dropDown_' + item.id + '" class="dropDown" onclick="toggleOpenCloseDropDown(\'dropDown_' + item.id + '\');"  onblur="toggleOpenCloseDropDown(\'dropDown_' + item.id + '\');" style="display: block;">';

				// set current closed value
				str = str + '<div class="dropDownClosed"><div class="dropDownLeft">' + item.innerHTML.strip() + '</div><div class="dropDownRight"></div></div>';

				str = str + '<div class="dropDownOpen"><div class="dropDownOpenLeft"><div class="dropDownOpenTop"><div class="dropDownOpenContent"><ul>';

				for (var i = 0; i < optionVals.length; i++) {
					var translatedOptionVal = Translator.translate(optionVals[i]);
					str = str + '<li onclick="changeInputDropDownValue(\'' + item.id + '\',\'' + translatedOptionVal + '\'); return true;"';
					// highlight current value
					if (translatedOptionVal == item.innerHTML.strip()) {
						str = str + ' class="on"';
					}
					str = str + '>' + translatedOptionVal + '</li>';
				}

				str = str + '</ul><div class="clear"></div></div></div></div></div></div>';

				// add hidden input field that will hold drop down's value
				str = str + '<input type="hidden" id="hiddenInput_' + item.id + '" name="' + item.id + '" value="' + item.innerHTML.strip() + '">';

				str = str + '</span>';

				item.replace(str);
			}
		}
	}

}

// replaces drop down with span
function inputToDropdownSpan(item) {

	if (item) {
		// pull value from hidden input field
		var value = item.down('input').getValue();

		var str = '<span id="' + item.id + '" class="inputFldDropdown" optionvals="' + item.readAttribute('optionvals') + '">' + value + '</span>';

		item.replace(str);
	}

}


// toggle section edit mode
function toggleSectionEditMode(sectionId) {

	if ($(sectionId)) {

		// if enabling edit mode
		if (!$(sectionId).hasClassName('editMode')) {
			// switch specific span classes to their corresponding input fields
			$(sectionId).select('tr.newRow span.inputFldDefault').each(function (item) { spanToDefaultInput(item); });
			$(sectionId).select('span.inputFldPassword').each(function (item) { spanToPasswordInput(item); });
			$(sectionId).select('span.inputFldText').each(function (item) { spanToTextInput(item); });
			$(sectionId).select('span.inputFldTextArea').each(function (item) { spanToTextAreaInput(item); });
			$(sectionId).select('span.inputFldRadio').each(function (item) { spanToRadioInput(item); });
			$(sectionId).select('span.inputFldCheckbox').each(function (item) { spanToCheckboxInput(item); });
			$(sectionId).select('span.inputFldDropdown').each(function (item) { spanToDropdownInput(item); });
			$(sectionId).select('span.inputFldDisabled').each(function (item) { if (item.up('tr')) { item.up('tr').addClassName('disabled'); } });
			$(sectionId).select('span.inputFldIpAddress').each(function (item) { formUtils.ipFields.spanToInput(item); });
			$(sectionId).select('span.inputFldTimeScroller').each(function(item) { spanToTimeScroller(item); });

			// if drawer table, highlight row frame
			if ($(sectionId).up('tr')) {
				// add row edit mode class to row
				$(sectionId).up('tr').toggleClassName('rowEditMode');

				// if two previous rows exist (because first sibling row is its closed state row, so skip), 
				// add highlighted top class to row - causes it to turn orange
				if ($(sectionId).up('tr').previousSiblings().size() > 1) {
					$(sectionId).up('tr').previousSiblings()[1].addClassName('highlightedTop');
				}

				// if next row exists, add highlighted bottom class to row - causes it to turn orange
				if ($(sectionId).up('tr').nextSiblings().size() > 0) {
					$(sectionId).up('tr').nextSiblings()[0].addClassName('highlightedBottom');
				}
			}

			// if drawer table, disable all other plus buttons and highlight row frame
			if ($(sectionId).up('table.drawerTable')) {
				// cycle through all other rows, to disable their plus/minus and edit buttons
				$(sectionId).up('table.drawerTable').select('.oddRow', '.evenRow', '.drawerExpandedRow').each(function (item) {
					if (item.down('.drawerNavButton')) {
						// disabled all plus/minus buttons
						item.down('.drawerNavButton').addClassName('disabled');
					}
					if (!item.id == $(sectionId).up('tr.drawerExpandedRow').id) {
						// also, for the rows not in edit mode, disable entire row, and section menu, if exists
						item.addClassName('disabled');
						item.select('.containerSection').each(function (section) { disableSectionMenu(section.id); });
					}
				});
			}

			// if in tab content area, disable other tabs
			if ($(sectionId).up('.containerTabContent') && $(sectionId).up('.containerTabContent').previous('.containerTabs')) {
				// cycle through all tabs, and if not in on state, add disabled class
				$(sectionId).up('.containerTabContent').previous('.containerTabs').select('.containerTab').each(function (item) { if (!item.hasClassName('onTab')) { item.addClassName('disabled'); } });
			}

			// disable section menu
			disableSectionMenu(sectionId);

			// disable all edit mode buttons throughout page
			$$('.sectionEditButton').each(function (item) { item.addClassName('disabled'); });

			// disable container min/max button, if exists
			if ($(sectionId).up('.container') && $(sectionId).up('.container').down('.minMaxButton')) {
				$(sectionId).up('.container').down('.minMaxButton').addClassName('disabled');
			}
		} else {
			// switch specific input fields back to their corresponding spans with values
			$(sectionId).select('tr.newRow input.inputFldDefault').each(function (item) { inputToDefaultSpan(item); });
			$(sectionId).select('input.inputFldPassword').each(function (item) { inputToPasswordSpan(item); });
			$(sectionId).select('input.inputFldText').each(function (item) { inputToTextSpan(item); });
			$(sectionId).select('textarea.inputFldTextArea').each(function (item) { inputToTextAreaSpan(item); });
			$(sectionId).select('span.inputFldRadioSpan').each(function (item) { inputToRadioSpan(item); });
			$(sectionId).select('span.inputFldCheckboxSpan').each(function (item) { inputToCheckboxSpan(item); });
			$(sectionId).select('span.inputFldDropdownSpan').each(function (item) { inputToDropdownSpan(item); });
			$(sectionId).select('span.inputFldDisabled').each(function (item) { if (item.up('tr')) { item.up('tr').removeClassName('disabled'); } });
			$(sectionId).select('input.inputFldIpAddress').each(function(item) { formUtils.ipFields.inputToSpan(item); });

			// if drawer table, unhighlight row frame
			if ($(sectionId).up('tr')) {
				// remove row edit mode class to row
				$(sectionId).up('tr').toggleClassName('rowEditMode');

				// if two previous rows exist (because first sibling row is its closed state row, so skip), 
				// remove highlighted top class to row - causes it to return to default grey
				if ($(sectionId).up('tr').previousSiblings().size() > 1) {
					$(sectionId).up('tr').previousSiblings()[1].removeClassName('highlightedTop');
				}

				// if next row exists, remove highlighted bottom class to row - causes it to return to default grey
				if ($(sectionId).up('tr').nextSiblings().size > 0) {
					$(sectionId).up('tr').nextSiblings()[0].removeClassName('highlightedBottom');
				}
			}

			// if drawer table, reenable all other plus buttons
			if ($(sectionId).up('table.drawerTable')) {
				// cycle through all other rows, to enable their plus/minus and edit buttons
				$(sectionId).up('table.drawerTable').select('.oddRow', '.evenRow', '.drawerExpandedRow').each(function (item) {
					if (item.down('.drawerNavButton')) {
						item.down('.drawerNavButton').removeClassName('disabled');
					}
					item.removeClassName('disabled');
					item.select('.containerSection').each(function (section) { enableSectionMenu(section.id); });
				});
			}

			// reenable all plus/minus buttons
			$(sectionId).select('.drawerNavButton').each(function (item) { item.removeClassName('disabled'); });

			// reenable all rows
			$(sectionId).select('tr').each(function (item) { item.removeClassName('disabled'); });

			// if in tab content area, enable other tabs
			if ($(sectionId).up('.containerTabContent') && $(sectionId).up('.containerTabContent').previous('.containerTabs')) {
				$(sectionId).up('.containerTabContent').previous('.containerTabs').select('.containerTab').each(function (item) { if (!item.hasClassName('onTab')) { item.removeClassName('disabled'); } });
			}

			// if they exist, hide selection checkboxes column - only displayed using special section task mode
			if ($(sectionId).select('td.selectionCol', 'th.selectionCol')) {
				$(sectionId).select('td.selectionCol', 'th.selectionCol').each(function (item) { item.hide(); });
			}

			// if they exist, hide deletion checkboxes column - only displayed using special section task mode
			if ($(sectionId).select('td.deleteCol', 'th.deleteCol')) {
				$(sectionId).select('td.deleteCol', 'th.deleteCol').each(function (item) { item.hide(); });
			}

			// always remove special edit class - only added using special section task mode
			$(sectionId).removeClassName('specialTaskMode');

			// always reset text of apply button
			$(sectionId).select('.stdButtonEmphasized1 span').each(function (item) { item.update(Translator.translate('generic.apply')); });

			// enable section menu
			enableSectionMenu(sectionId);

			// enable all edit mode buttons throughout page
			$$('.sectionEditButton').each(function (item) { item.removeClassName('disabled'); });

			// enable container min/max button
			if ($(sectionId).up('.container') && $(sectionId).up('.container').down('.minMaxButton')) {
				$(sectionId).up('.container').down('.minMaxButton').removeClassName('disabled');
			}


			// hide task button, if exists - only set using special section task mode
			if ($(sectionId).down('.sectionTaskButton')) {
				$(sectionId).down('.sectionTaskButton').hide();
			}

			// show task menu, if exists - only set using special section task mode
			if ($(sectionId).down('.sectionTaskMenu')) {
				$(sectionId).down('.sectionTaskMenu').show();
			}


			// unset specialSectionTask item, even if not used
			specialSectionTaskType = null;
			specialSectionTaskUrl = null;

			// uncheck all row selection checkboxes
			if ($(sectionId).down('.selectAllRows')) {
				$(sectionId).down('.selectAllRows').checked = false;
				$(sectionId).select('.selectAllRowsItem').each(function (item) { item.checked = false; toggleRowHighlight(item); });
			}


			// uncheck all row deletion checkboxes
			$(sectionId).select('.deleteRowItem').each(function (item) { item.checked = false; toggleRowHighlight(item); });

		}

		// toggle sections edit mode class
		$(sectionId).toggleClassName('editMode');

		// if contains scrollable area, re-setup scrollbar
		if ($(sectionId).down('.scrollAreaContainer')) {
			setupScrollArea($(sectionId).down('.scrollAreaContainer'));
		}
		// if contained within scrollable area, re-setup scrollbar
		if ($(sectionId).up('.scrollAreaContainer')) {
			setupScrollArea($(sectionId).up('.scrollAreaContainer'));

			// see if needing to toggle highlight row sides
			if ($(sectionId).up('tr')) {
				setupScrollAreaHighlightRowSides($(sectionId).up('.scrollAreaContainer'), $(sectionId).up('tr'));
			}

			disableScrollbar($(sectionId).up('.scrollAreaContainer'));
		}

		// if inside modal content window, redo scroll area
		if ($(sectionId).up('#modalContent')) {
			setModalContentDimensions();
		}
	}

}

var inModalClose = false;

// apply edit mode ie. submit button clicked
function applyEditMode(sectionId, functionName, onFinish) {

	if ($(sectionId)) {

		// special process if modal window needing to close - either wizard modal or specified auto close modal
		if ($(sectionId).up('#modalContent')
				&& ($(sectionId).up('.wizardContainer') || $(sectionId).up('.modalAutoCloseOnApplyOrCancel'))) {
			inModalClose = true;
		}

		// special case where section within table drawer is in edit mode, change section id value to inner edit mode section
		if ($(sectionId).select('.editMode').size() > 0) {
			sectionId = $(sectionId).select('.editMode')[0].id;
		}



		// temporary display form values set ****for testing only
		var tmpMsg = '';
		if (specialSectionTaskUrl) {
			tmpMsg = 'would now procede to special task:\n' + specialSectionTaskUrl + '\n';
		} else {
			tmpMsg = 'would now save the following values:\n';
		}

		if ($(sectionId).down('form')) {
			var form = $(sectionId).down('form');
		} else if ($(sectionId).up('form')) {
			var form = $(sectionId).up('form');
		}

		if (form) {
			if (functionName != null) {
				//if( formUtils.validate.check(sectionId) == true ){
				eval(functionName(form));
				//}
			} else {
				tmpMsg = tmpMsg + '\nvalues:\n' + unescape(form.serialize()).replace(/&/g, '\n');
				//alert(tmpMsg);
			}
			//tmpMsg = tmpMsg + '\nvalues:\n' + unescape(form.serialize()).replace(/&/g, '\n');
		}
		//alert(tmpMsg);


		// if add new row, reset table
		// NOTE: this is incomplete in that it does not another hidden new row
		if (specialSectionTaskType == 'row-create-and-deletion') {

			// if exists, show green star column
			if ($(sectionId).select('tr.newRow').size() > 0) {

				$(sectionId).select('td.starCol', 'th.starCol').each(function (item) { item.show(); });

				// change remove all firstCol class names
				$(sectionId).select('td.firstCol', 'th.firstCol').each(function (item) { item.removeClassName('firstCol'); });

				// set new row to show green star
				$(sectionId).select('tr.newRow').each(function (item) { item.addClassName('showGreenStar'); });
			}

			// if contained within tab, green star tab
			if ($(sectionId).up('.containerTabContent') && $(sectionId).up('.containerTabContent').readAttribute('tabid') != '') {
				// tab_0_0 -> tab_0_0_tab
				var tmpTabId = $(sectionId).up('.containerTabContent').readAttribute('tabid') + '_tab';

				if ($(tmpTabId)) {
					$(tmpTabId).addClassName('showGreenStar');
				}
			}
		}


		// first, display success message
		var sectionMessagesId = sectionId + '_messages';
		// only show success message if not in modal close section, and section has a message boxes container (which they all should have)
		/*if (!inModalClose && $(sectionMessagesId)) {
		// need to randomly generate a unique id
		var tmpId = Math.floor(Math.random() * 10000);
		tmpId = sectionMessagesId + '_' + tmpId;

		var msgText = 'You have successfully ';
		if (specialSectionTaskUrl) {
		msgText = msgText + '\'' + specialSectionTaskUrl + '\' to ';
		} else {
		msgText = msgText + 'edited ';
		}
		msgText = msgText + 'this section.';

		// insert new success message box at bottom of existing message boxes
		$(sectionMessagesId).insert({bottom: '<div id="' + tmpId + '" class="msgBox msgSuccess"><div class="msgIcon"></div><div class="msgContent"><table class="msgTable"><tr><td><div class="msgTitle">Congratulations.</div></td><td class="maxWidthCol"><div class="msgText">' + msgText + '</div></td></tr></table></div><div class="msgCloseButton" onclick="closeMsgBox(\'' + tmpId + '\');"></div></div>'});
		}*/

		//        if (inModalClose) {
		//            // close modal
		//            closeModal();
		//        }

		// no edit mode toggling for modal close
		if (!inModalClose) {
			// next, coninue to toggle edit mode
			toggleSectionEditMode(sectionId);
			if (onFinish != null) {
				onFinish();
			} else {
				toggleSectionEditMode(sectionId);
			}
		}
	}
}

// reset edit mode form
function resetSectionEditForm(sectionId) {

	if ($(sectionId)) {
		// special case if within drawer expanded view section - must go up to container level to find form
		if ($(sectionId).hasClassName('tableRowExpandedViewSection')) {
			// reset form values
			$(sectionId).up('.containerSection').select('form').each(function (item) { item.reset(); });
		} else {
			// reset form values
			$(sectionId).select('form').each(function (item) { item.reset(); });
		}

		// if add new row, reset table
		if (specialSectionTaskType == 'row-create-and-deletion') {
			// if exists, hide new row
			$(sectionId).select('tr.newRow').each(function (item) { item.hide(); });
			// change all the other row classes
			$(sectionId).select('tr').each(function (item) {
				if (item.hasClassName('oddRow')) {
					item.addClassName('evenRow');
					item.removeClassName('oddRow');
				} else if (item.hasClassName('evenRow')) {
					item.addClassName('oddRow');
					item.removeClassName('evenRow');
				}
			});
		}
	}
}

// entering edit mode by clicking on edit button
function editButtonClick(sectionId) {

	if ($(sectionId)) {

		// check to see if button doesn't contain disabled class
		if (!$(sectionId).down('.sectionEditButton').hasClassName('disabled')) {

			// if enabling edit mode
			if (!$(sectionId).hasClassName('editMode')) {
				// first, auto turn off any other edit modes across the entire page
				$$('.editMode').each(function (item) { toggleSectionEditMode(item.id); });

				// next, auto close all success and warning info boxes in container, only if they contain a close button
				$(sectionId).select('.msgSuccess', '.msgFailure').each(function (item) { if (item.down('.msgCloseButton')) { closeMsgBox(item.id); } });

				// if disabling edit mode
			} else {
				// reset form values **not saving values
				resetSectionEditForm(sectionId);
			}


			// next, toggle edit mode
			toggleSectionEditMode(sectionId);
		}
	}

}

// edit mode cancel button clicked
function cancelEditMode(sectionId) {

	if ($(sectionId)) {
		// if modal and in specified auto close modal, just close modal
		var testCondition1 = $$('#modalContent #' + sectionId);
		var testCondition2 = $$('.modalAutoCloseOnApplyOrCancel #' + sectionId);
		if (testCondition1.length > 0 && testCondition2.length > 0) {
			// close modal
			closeModal();

		} else {

			// first, reset form values
			resetSectionEditForm(sectionId);

			// next, toggle edit mode
			toggleSectionEditMode(sectionId);

		}
	}

}


// special section task modes - row selection, row deletion, or row create and deletion
function specialSectionTaskMode(sectionId, type, taskTitle, taskUrl) {

	if ($(sectionId)) {
		// special set up by given type
		if (type == 'row-selection') {
			// display selection checkboxes column
			$(sectionId).select('td.selectionCol', 'th.selectionCol').each(function (item) { item.show(); });

			// disable any rows where the checkbox is disabled
			$(sectionId).select('input.selectAllRowsItem').each(function (item) { if (item.disabled && item.up('tr')) { item.up('tr').addClassName('disabled'); } });

			// change apply button to 'next', using extraInfo as where to go next
			$(sectionId).select('.stdButtonEmphasized1 span').each(function (item) { item.update(Translator.translate('generic.next')); });

			// disable all plus/minus buttons
			$(sectionId).select('.drawerNavButton').each(function (item) { item.addClassName('disabled'); });

			// make sure any contained drawer tables have an auto table layout - this allows for shifting
			$(sectionId).select('.drawerTable').each(function (item) { item.setStyle({ 'tableLayout': 'auto' }); });

		} else if (type == 'row-deletion') {
			// change apply button to 'delete', using extraInfo as where to go next
			$(sectionId).select('.stdButtonEmphasized1 span').each(function (item) { item.update(Translator.translate('generic.delete')); });

		} else if (type == 'row-create-and-deletion') {
			// display new row
			$(sectionId).select('tr.newRow').each(function (item) { item.show(); });

			// change all the other row classes
			$(sectionId).select('tr').each(function (item) {
				if (item.hasClassName('oddRow')) {
					item.addClassName('evenRow');
					item.removeClassName('oddRow');
				} else if (item.hasClassName('evenRow')) {
					item.addClassName('oddRow');
					item.removeClassName('evenRow');
				}
			}); ;
		}

		if (type == 'row-deletion' || type == 'row-create-and-deletion') {
			// display delete checkboxes column
			$(sectionId).select('td.deleteCol', 'th.deleteCol').each(function (item) { item.show(); });

			// disable any rows where the checkbox is disabled
			$(sectionId).select('input.deleteRowItem').each(function (item) { if (item.disabled && item.up('tr')) { item.up('tr').addClassName('disabled'); } });

			// disabled all plus/minus buttons
			$(sectionId).select('.drawerNavButton').each(function (item) { item.addClassName('disabled'); });

			// make sure any contained drawer tables have an auto table layout
			$(sectionId).select('.drawerTable').each(function (item) { item.setStyle({ 'tableLayout': 'auto' }); });
		}


		// hide task menu drop down, if exists
		if ($(sectionId).down('.sectionTaskMenu')) {
			$(sectionId).down('.sectionTaskMenu').hide();
		}

		// show task button, if exists - after first updating its display text
		if ($(sectionId).down('.sectionTaskButton')) {
			$(sectionId).down('.sectionTaskButton').down('.sectionTaskButtonInner').update(taskTitle);
			$(sectionId).down('.sectionTaskButton').show();
		}

		// set variables to remember special task settings
		specialSectionTaskType = type;
		specialSectionTaskUrl = taskUrl;

		$(sectionId).addClassName('specialTaskMode');

		// now run standard edit button click
		editButtonClick(sectionId);
	}

}


// change highlighted item in drop down list
function changeSelectedItemInDropDownList(dropDown, selectedItemText) {

	if (dropDown) {
		// change highlighted menu item
		dropDown.select('li').each(function (item) { item.removeClassName('on'); if (item.innerHTML.strip() == selectedItemText) { item.addClassName('on'); } });
	}

}


// make change to select view drop down item
function changeSelectView(sectionId, selectedItemText, elementId, argsArr) {

	if ($(sectionId)) {
		// change highlighted menu item, if it exists - does not change close value
		if ($(sectionId).down('.sectionSelectViewMenu')) {
			changeSelectedItemInDropDownList($(sectionId).down('.sectionSelectViewMenu'), selectedItemText);
		}

		// then, change whatever content needs to be updated
		if ($(sectionId).down(elementId)) {
			if (argsArr.length == 2) {
				$(sectionId).down(elementId).writeAttribute(argsArr[0], argsArr[1]);
			} else {
				$(sectionId).down(elementId).update(argsArr[0]);
			}

			// adjust scrollbars, if need be
			if ($(sectionId).down(elementId).up('.scrollAreaContainer')) {
				// slight delay to allow image to load
				window.setTimeout(function () {
					setupScrollArea($(sectionId).down(elementId).up('.scrollAreaContainer'));
				},
							200);
			}
		}
	}

}

// change drop down input's value
function changeInputDropDownValue(id, selectedItemText) {

	inputId = 'hiddenInput_' + id;
	dropDownId = 'dropDown_' + id;

	if ($(inputId) && $(dropDownId)) {
		// change highlighted menu item
		changeSelectedItemInDropDownList($(dropDownId), selectedItemText);

		// then, update drop down closed value
		if ($(dropDownId).down('.dropDownLeft')) {
			$(dropDownId).down('.dropDownLeft').update(selectedItemText);

			// make sure doesn't become wider than contain fld column
			if ($(dropDownId).up('.fldValue')) {
				var maxWidth = $(dropDownId).up('.fldValue').getWidth() - 32;
				var tmpSelectedItemText = selectedItemText;
				// slowly remove text from the closed value until it fits within max width
				while ($(dropDownId).down('.dropDownLeft').getWidth() > maxWidth) {
					tmpSelectedItemText = tmpSelectedItemText.substring(0, (tmpSelectedItemText.length - 1));
					//  update drop down closed value until it fits
					$(dropDownId).down('.dropDownLeft').update(tmpSelectedItemText + '...');
				}
			}
		}

		// lastly, update hidden input field value
		$(inputId).value = selectedItemText;
	}

}



// toggles cell's containing row's highlight mode by toggling the 'highlight' class
function toggleHighlightRow(elementId) {

	if ($(elementId)) {
		if ($(elementId).up('tr')) {
			$(elementId).up('tr').toggleClassName('highlight');
		}
	}
}

//disable inputs if the given input's value does not start with the given text
function bindDisableInputsIfDoesNotStartsWith(elementId, startsWithText, inputIdsToConditionallyDisable) {

	function changeFunc() {
		var testInputValue = $F(elementId);

		var doesElementStartWith = testInputValue.startsWith(startsWithText);

		inputIdsToConditionallyDisable.each(function (conditionallyDisableElementId) {
			if (!doesElementStartWith) {
				$(conditionallyDisableElementId).disable();
				$(conditionallyDisableElementId).value = '';
			} else {
				$(conditionallyDisableElementId).enable();
			}
		});
	};


	$(elementId).observe('keyup', changeFunc);
	$(elementId).observe('blur', changeFunc);
	changeFunc();
}

// show tab
function showTab(tabId) {

	var tabContentId = tabId + '_content';
	var tabTabId = tabId + '_tab';

	if ($(tabContentId) && $(tabTabId) && !$(tabTabId).hasClassName('disabled')) {

		// first, change tab tab display by removing on tab class from all other tabs, then adding it to selected tab
		$(tabTabId).siblings().each(function (item) { item.removeClassName('onTab'); });

		$(tabTabId).addClassName('onTab');

		// second, hide other tabs' content, showing only the selected tab's content 
		$(tabContentId).siblings().each(function (item) {
			if (item.hasClassName('containerTabContent')) {
				item.hide();
			}
		});

		$(tabContentId).show();

		// if inside scrollable area, re-setup scrollbar
		if ($(tabContentId).down('.scrollAreaContainer')) {
			setupScrollArea($(tabContentId).down('.scrollAreaContainer'));
		}
	}

}

// scroll alert tab pagination controls
function updatelAlertTabPagination(tabContentId, pageCount, scrollWidth) {

	if ($(tabContentId)) {

		if ($(tabContentId).select('.alertTabPagesInner').size()) {

			// determine the current page based on how far the scroll area is already offset, divided by each pages default scroll width
			var curPage = Math.round((-($(tabContentId).select('.alertTabPagesInner')[0].positionedOffset().left) / scrollWidth) + 1);

			// update current page text span
			$(tabContentId).select('.sectionPaginationText')[0].select('span')[0].update(curPage);

			if (curPage > 1) {
				// enable paginate left control
				$(tabContentId).select('.sectionPaginationLeft')[0].removeClassName('disabled');
			} else {
				// disable paginate left control
				$(tabContentId).select('.sectionPaginationLeft')[0].addClassName('disabled');
			}

			if (curPage < pageCount) {
				// enable paginate right control
				$(tabContentId).select('.sectionPaginationRight')[0].removeClassName('disabled');
			} else {
				// disable paginate right control
				$(tabContentId).select('.sectionPaginationRight')[0].addClassName('disabled');
			}

		}
	}
}

// scroll alert tab content
function scrollAlertTabScroll(tabContentId, direction) {

	if ($(tabContentId)) {

		if ($(tabContentId).select('.alertTabPagesInner').size()) {

			// determine the total page count based on the scroll area's page count
			var pageCount = $(tabContentId).select('.alertTabPagesInner')[0].select('.alertTabPage').size();
			if (pageCount > 0) {
				// base default scroll width on width of individual page
				var scrollWidth = $(tabContentId).select('.alertTabPagesInner')[0].select('.alertTabPage')[0].getDimensions().width;
			} else {
				// default scroll width
				var scrollWidth = 585;
			}

			// animate scroll inner scroll area one scroll width, with direction -1 being to the right, and +1 being to the left
			new Effect.Move($(tabContentId).select('.alertTabPagesInner')[0], { duration: paginationScrollSpeed,
				transition: Effect.Transitions.sinoidal,
				x: scrollWidth * direction,
				y: 0,
				mode: 'relative',
				// when finished, update alert tab pagination controls
				afterFinish: function () { updatelAlertTabPagination(tabContentId, pageCount, scrollWidth); },
				// this prevents the scroll from occurring again until this one is completed
				queue: { scope: 'alertmove', limit: 1 }
			});
		}

	}

}

// scroll inner content to the right, if right control not disabled
function scrollAlertTabRight(tabContentId) {

	if (!$(tabContentId).select('.sectionPaginationRight')[0].hasClassName('disabled')) {
		scrollAlertTabScroll(tabContentId, -1);
	}

}

// scroll inner content to the left, if left control not disabled
function scrollAlertTabLeft(tabContentId) {

	if (!$(tabContentId).select('.sectionPaginationLeft')[0].hasClassName('disabled')) {
		scrollAlertTabScroll(tabContentId, 1);
	}

}


// close specific message box
function closeMsgBox(id) {
	if ($(id)) {
		// animate blind up
		$(id).blindUp({ duration: msgBoxCloseSpeed });

		// if inside scrollable area, re-setup scrollbar after a pause
		if ($(id).up('.scrollAreaContainer')) {
			window.setTimeout(function () {
				setupScrollArea($(id).up('.scrollAreaContainer'));
			},
							msgBoxCloseSpeed * 1100);
		}

		// if inside modal content window, redo scroll area after a pause
		if ($(id).up('#modalContent')) {
			window.setTimeout(function () {
				setModalContentDimensions();
			},
							msgBoxCloseSpeed * 1100);
		}
	}
}


// tool tip variables
var tooltipDelayId = null;
var tooltipDelayTime = 500; // in milliseconds


// delay showing tool tip
function delayShowTooltip(id) {

	if ($(id)) {
		// first, kill any other delayed tooltip timeouts
		if (tooltipDelayId) {
			window.clearTimeout(tooltipDelayId);
			tooltipDelayId = null;
		}

		// set timeout to delay showing tooltip
		tooltipDelayId = window.setTimeout(function () { showTooltip(id); }, tooltipDelayTime);
	}

}

// displays specified tooltip
function showTooltip(id) {

	if ($(id)) {
		$(id).show();
	}

}

// displays help **need implementation
function showHelp() {
	// nothing
	alert('coming soon');
}

// move tooltip to follow mouse move
function msgBoxWithTooltipOnMouseMove(e) {

	if (e.target && e.target.select('.msgTooltip').size()) {

		var tmp = e.target.select('.msgTooltip')[0];

		tmp.style.left = (Event.pointerX(e) - e.target.cumulativeOffset()[0] + 140) + 'px';
		tmp.style.bottom = (-Event.pointerY(e) + e.target.cumulativeOffset()[1] + 40) + 'px';

		delayShowTooltip(tmp.id);

	}

}

// hide specified tooltip
function hideTooltip(id) {

	if ($(id)) {
		// first, kill any delayed tooltip timeouts
		if (tooltipDelayId) {
			window.clearTimeout(tooltipDelayId);
			tooltipDelayId = null;
		}

		$(id).hide();
	}

}



// toggle open close drop down
function toggleOpenCloseDropDown(dropDownId) {

	if ($(dropDownId)) {
		// make sure drop down isn't disabled, or contained within a disabled section menu
		if (!$(dropDownId).hasClassName('disabled') && (!$(dropDownId).up('.sectionMenu') || !$(dropDownId).up('.sectionMenu').hasClassName('menuDisabled'))) {

			// opening drop down
			if (!$(dropDownId).hasClassName('opened')) {
				// close all open drop downs across entire page
				$$('.dropDown.opened').each(function (item) { closeDropDown(item.id); });

				// add opened class to drop down container
				$(dropDownId).addClassName('opened');

				// resize menu drop downs open width, if smaller than closed width - only for select view and task menu drop downs
				if ($(dropDownId).hasClassName('sectionSelectViewMenu') || $(dropDownId).hasClassName('sectionTaskMenu')) {
					if ($(dropDownId).getWidth() > ($(dropDownId).down('ul').getWidth() + 2)) {
						var minWidth = $(dropDownId).getWidth() - 2;
						$(dropDownId).down('ul').setStyle({ 'width': minWidth + 'px' });
					}
				}

				// IE7 bug fix: li widths are not 100% of ul, so they must be set here
				if (Prototype.Browser.IE && parseInt(navigator.userAgent.substring(navigator.userAgent.indexOf("MSIE") + 5)) == 7) {
					var liWidth = $(dropDownId).down('ul').getWidth() - 36;
					$(dropDownId).down('ul').select('li').each(function (item) { item.setStyle({ 'width': liWidth + 'px' }); });
				}


				// if contained in data field row, add up class to fix z-index, and make sure drop down is opened on top of any other content
				if ($(dropDownId).up('.dataFieldRow')) {
					$(dropDownId).up('.dataFieldRow').addClassName('up');
				}

			} else {
				// call close drop down
				closeDropDown(dropDownId);
			}
		}
	}

}

// close specified drop down
function closeDropDown(dropDownId) {

	if ($(dropDownId)) {
		// remove opened class to drop down container
		$(dropDownId).removeClassName('opened');

		// if contained in data field row, remove up class to bring layer back to same standard level
		if ($(dropDownId).up('.dataFieldRow')) {
			$(dropDownId).up('.dataFieldRow').removeClassName('up');
		}
	}

}



// open drawer table expanded row
function openDrawerExpandedView(rowId) {

	var expandedRowId = rowId + '_expanded';

	if ($(rowId) && $(expandedRowId)) {

		// disable if drawerNavButton is disabled
		if (!$(rowId).down('.drawerNavButton').hasClassName('disabled')) {

			// special case with alert settings nested tables
			var expandedNestedRowId = rowId + '_expanded_nested';
			if ($(expandedNestedRowId)) {
				$(expandedNestedRowId).show();
				// and padding row
				var expandedNestedRowPaddingId = rowId + '_expanded_nested_padding';
				if ($(expandedNestedRowPaddingId)) {
					$(expandedNestedRowPaddingId).show();
				}

				// enable table headers
				enableTableHeaders($(rowId).up('table').id);

				// special case with nested tables
			} else if ($(expandedNestedRowId + '_0')) {
				// cycle through all the hidden nested rows
				var nestedRowCount = 0;
				while ($(expandedNestedRowId + '_' + nestedRowCount)) {
					$(expandedNestedRowId + '_' + nestedRowCount).show();
					nestedRowCount++;
				}

				// enable table headers
				enableTableHeaders($(rowId).up('table').id);

			} else {
				// non-nested drawer table - columns need to be fixed in width
				// only set the col widths once
				if ($(rowId).up('table').getStyle('tableLayout') != 'fixed') {
					// set all the col widths and change table layout to fixed, to prevent shifting
					// **be sure to skip internal table row headers 
					$(rowId).up('table').select('th').each(function (item) { if (item.visible() && !item.up('td')) { item.setStyle({ 'width': (item.getWidth() - (getValueFromStyle(item.getStyle('padding-left')) + getValueFromStyle(item.getStyle('padding-right')))) + 'px' }); } });
					$(rowId).up('table').setStyle({ 'tableLayout': 'fixed' });
				}

				// disable section menu and table headers
				disableSectionMenu($(rowId).up('.containerSection').id);
				disableTableHeaders($(rowId).up('table').id);

			}


			// hide compressed row
			$(rowId).hide();
			// show expanded row
			$(expandedRowId).show();
			// animate blind down all expanded content containers
			$(expandedRowId).select('.drawerExpandedContentContainer').each(function (item) { item.blindDown({ duration: drawerOpenCloseSpeed }); });

			// after proper delay
			window.setTimeout(function () {
				// reload scroll area
				setupScrollArea($(rowId).up('.scrollAreaContainer'));
				// set all contained scroll areas
				if ($(expandedRowId).select('.scrollAreaContainer')) {
					$(expandedRowId).select('.scrollAreaContainer').each(function (child) { setupScrollArea(child); });
				}

				// if not nested drawer table, disable scroll bar
				if (!$(expandedNestedRowId + '_0')) {
					// disable scroller
					disableScrollbar($(rowId).up('.scrollAreaContainer'));
				}

				// if inside modal content window, redo scroll area
				if ($(rowId).up('#modalContent')) {
					setModalContentDimensions();
				}

			}, drawerOpenCloseSpeed * 1100);

		}
	}

}


// close drawer table expanded row
function closeDrawerExpandedView(rowId) {

	var expandedRowId = rowId + '_expanded';

	if ($(rowId) && $(expandedRowId)) {

		// disable if drawerNavButton is disabled
		if (!$(rowId).down('.drawerNavButton').hasClassName('disabled')) {

			// if in edit mode, cancel edit mode
			if ($(expandedRowId).select('.editMode').size()) {
				$(expandedRowId).select('.editMode').each(function (item) { cancelEditMode(item.id) });
			}

			// animate blind up all expanded content containers
			$(expandedRowId).select('.drawerExpandedContentContainer').each(function (item) { item.blindUp({ duration: drawerOpenCloseSpeed }); });

			// after proper delay
			window.setTimeout(function () {
				// special case with alert settings nested tables
				var expandedNestedRowId = rowId + '_expanded_nested';
				if ($(expandedNestedRowId)) {
					$(expandedNestedRowId).hide();
					// and padding row
					var expandedNestedRowPaddingId = rowId + '_expanded_nested_padding';
					if ($(expandedNestedRowPaddingId)) {
						$(expandedNestedRowPaddingId).hide();
					}

					// special case with nested tables
				} else if ($(expandedNestedRowId + '_0')) {
					// cycle through all the hidden nested rows
					var nestedRowCount = 0;
					while ($(expandedNestedRowId + '_' + nestedRowCount)) {
						$(expandedNestedRowId + '_' + nestedRowCount).hide();
						nestedRowCount++;
					}
				}

				// hide expanded row
				$(expandedRowId).hide();
				// show compressed row
				$(rowId).show();

				// reload scroll area
				setupScrollArea($(rowId).up('.scrollAreaContainer'));

				// if no other drawers are expanded, change table settings
				var $openDrawers = false;
				// cycle through all expanded rows to determine if any are visible
				$(rowId).up('.containerSection').select('.drawerExpandedRow').each(function (item) { if (item.visible()) { $openDrawers = true; } });
				if (!$openDrawers) {
					if ($(expandedNestedRowId) || $(expandedNestedRowId + '_0')) {
						// disable table headers
						disableTableHeaders($(rowId).up('table').id);

					} else {
						// enable section menu and table headers
						enableSectionMenu($(rowId).up('.containerSection').id);
						enableTableHeaders($(rowId).up('table').id);
					}
				} else {
					// keep scroller disabled
					if (!$(expandedNestedRowId) && !$(expandedNestedRowId + '_0')) {
						// disable scroller
						disableScrollbar($(rowId).up('.scrollAreaContainer'));
					}
				}


				// if inside modal content window, redo scroll area
				if ($(rowId).up('#modalContent')) {
					setModalContentDimensions();
				}


			}, drawerOpenCloseSpeed * 1100);
		}
	}
}


// changes alert settings nested items
function changeDrawerExpandedNestedItem(itemId) {

	var itemNavId = itemId + '_navitem';

	if ($(itemNavId)) {

		// turn off all other nested nav items
		$(itemNavId).siblings().each(function (item) { item.removeClassName('on'); });

		// turn on specific nested nav item
		$(itemNavId).addClassName('on');

		// hide all nested data columns
		$(itemNavId).up('tr').select('.nestedTableColumn').each(function (item) { item.hide(); });

		// show only current item's nested data columns
		var tmpCount = 0;
		while ($(itemId + '_' + tmpCount)) {
			$(itemId + '_' + tmpCount).show();
			tmpCount++;
		}

		// if inside scrollable area, re-setup scrollbar
		if ($(itemNavId).up('.scrollAreaContainer')) {
			setupScrollArea($(itemNavId).up('.scrollAreaContainer'));
		}
	}

}


// toggle select all checkbox
function toggleSelectAllRows(tableId) {

	if ($(tableId) && $(tableId).down('.selectAllRows')) {
		// check/uncheck all boxes based on specific checkbox value
		$(tableId).select('.selectAllRowsItem').each(function (item) { if (!item.disabled) { item.checked = $(tableId).down('.selectAllRows').checked; toggleRowHighlight(item); } });
	}

}


// toggle selected row item
function toggleSelectRowItems(tableId) {

	if ($(tableId) && $(tableId).down('.selectAllRows')) {
		// see if all specific checkboxes are checked, and highlight selected rows
		var allSelected = true;
		$(tableId).select('.selectAllRowsItem').each(function (item) { if (!item.disabled && !item.checked) { allSelected = false; }; oggleRowHighlight(item); });

		// set select all checkbox based on if all checkboxes were selected
		$(tableId).down('.selectAllRows').checked = allSelected;
	}

}

// toggle row highlight
function toggleRowHighlight(rowSelectionCheckbox) {

	if (rowSelectionCheckbox && rowSelectionCheckbox.up('tr')) {
		// highlight row by adding 'highlight' class, if row selection checkbox is checked
		if (rowSelectionCheckbox.checked) {
			rowSelectionCheckbox.up('tr').addClassName('highlight');
		} else {
			rowSelectionCheckbox.up('tr').removeClassName('highlight');
		}
	}

}


var slidersArr = [];


// setup table scroll area row highlight sides - orange table broder
function setupScrollAreaHighlightRowSides(item, row) {

	if (item && row) {
		var overlayLeft = item.down('.tableHighlightOverlaySideLeft');
		var overlayRight = item.down('.tableHighlightOverlaySideRight');

		if (overlayLeft && overlayRight) {
			// is row in edit mode
			if (row.hasClassName('rowEditMode')) {

				// show side highlights
				overlayLeft.show();
				overlayRight.show();
				// set positions and heights, with slight adjustments based on border widths
				var top = row.positionedOffset()[1] - 4;
				var height = row.getHeight() + 8;

				// special adjustments if first or last row
				if ((row.previousSiblings().size() > 1)
						&& row.previousSiblings()[1].down('.tableTop')
						&& item.down('.tableOverlayTop')) {
					// highlight table overlay top
					item.down('.tableOverlayTop').toggleClassName('highlighted');
					// adjust side height and positioning
					top = top + 4;
					height = height - 4;
				}
				if ((row.nextSiblings().size() > 0)
						&& row.nextSiblings()[0].down('.tableBottom')
						&& item.down('.tableOverlayBottom')) {
					// highlight table overlay bottom
					item.down('.tableOverlayBottom').toggleClassName('highlighted');
					// adjust side height
					height = height - 4;
				}

				overlayLeft.setStyle({ top: top + 'px', height: height + 'px' });
				overlayRight.setStyle({ top: top + 'px', height: height + 'px' });

			} else {
				// hide side highlights
				overlayLeft.hide();
				overlayRight.hide();

				// special adjustments if first or last row
				if ((row.previousSiblings().size() > 1)
						&& row.previousSiblings()[1].down('.tableTop')
						&& item.down('.tableOverlayTop')) {
					// unhighlight table overlay top
					item.down('.tableOverlayTop').toggleClassName('highlighted');
				}
				if ((row.nextSiblings().size() > 0)
						&& row.nextSiblings()[0].down('.tableBottom')
						&& item.down('.tableOverlayBottom')) {
					// unhighlight table overlay bottom
					item.down('.tableOverlayBottom').toggleClassName('highlighted');
				}
			}
		}
	}

}

// setup horizontal scroll area, as well as standard table border overlays
function setupScrollArea(item) {

	var scrollbarId = item.id + '_scrollbar_track';

	// make sure needed divs exist, and item is displayed ie. width > 0
	if (item && (item.getWidth() > 0) && item.down('.scrollAreaInnerContainer') && item.down('.scrollAreaContent') && $(scrollbarId)) {
		// reset area content position
		item.down('.scrollAreaContent').setStyle({ left: '0px' });

		var overlayTop = item.down('.tableOverlayTop');
		var overlayLeft = item.down('.tableOverlaySideLeft');
		var overlayRight = item.down('.tableOverlaySideRight');

		// position the table top and side border divs
		if (overlayTop && overlayLeft && overlayRight) {
			var maxHeight = 0;
			// determine the headers height based on the tallest header cell
			item.select('th').each(function (tmp) { if (tmp.visible() && tmp.getHeight() > maxHeight) { maxHeight = tmp.getHeight(); } });

			// show top border overlay, positioned just below header row
			overlayTop.show();
			overlayTop.setStyle({ top: (maxHeight - 4) + 'px' });

			// position and size side border overlays based on header row offset, and content total height
			var sidesHeight = item.down('.scrollAreaContent').getHeight() - maxHeight - 6;
			overlayLeft.setStyle({ top: maxHeight + 'px', height: sidesHeight + 'px' });
			overlayRight.setStyle({ top: maxHeight + 'px', height: sidesHeight + 'px' });
		}

		// determine if scroll area content is larger than its container
		if (item.down('.scrollAreaContent').getWidth() > item.down('.scrollAreaInnerContainer').getWidth()) {
			// show scrollbar track and handle
			$(scrollbarId).show();
			$(scrollbarId).removeClassName('disabled');
			$(scrollbarId).down('.scrollBarHandle').show();

			// destroy slider object, if it already exists
			if (slidersArr[item.id]) {
				slidersArr[item.id].dispose();
				slidersArr[item.id] = null;
			}

			// setup slider object
			slidersArr[item.id] = new Control.Slider($(scrollbarId).down('.scrollBarHandle'), $(scrollbarId), {
				// set range to match difference in scroll area content width versus its container width
				range: $R(0, (item.down('.scrollAreaContent').getWidth() - item.down('.scrollAreaInnerContainer').getWidth())),
				// reset slider to zero/left
				sliderValue: 0,
				onSlide: function (value) {
					item.down('.scrollAreaContent').setStyle({ left: -value + 'px' });
				},
				onChange: function (value) {
					item.down('.scrollAreaContent').setStyle({ left: -value + 'px' });
				}
			});

		} else {
			// hide scrollbar
			$(scrollbarId).hide();
		}
	}

}

// disabled the specified scrollbar
function disableScrollbar(item) {

	var scrollbarId = item.id + '_scrollbar_track';

	// make sure needed divs exist, and item is displayed ie. width > 0
	if (item && (item.getWidth() > 0) && item.down('.scrollAreaInnerContainer') && item.down('.scrollAreaContent') && $(scrollbarId)) {
		// reset area content position to zero/left
		item.down('.scrollAreaContent').setStyle({ left: '0px' });

		// disable slider, if it already exists, and reset position to zero/left
		if (slidersArr[item.id]) {
			slidersArr[item.id].setValue(0);
			slidersArr[item.id].setDisabled();

			$(scrollbarId).addClassName('disabled');

			// hide scrollbar handle
			if ($(scrollbarId).down('.scrollBarHandle')) {
				$(scrollbarId).down('.scrollBarHandle').hide();
			}
		}
	}

}


// modal popup window positioning variables
var modalTopOffset = 100;
var modalBottomOffset = 100;

var modalTopBottomHeight = 130;

var modalScrollbar = null;


// set modal overlay height and popup window top position based on browser window (viewport) dimensions and scroll offset
function setInitialModalDimensions() {

	if ($('main') && $('modalOverlay') && $('modalOuterContainer') && $('modalInnerContainer') && $('modalScrollbarContainer')) {

		var viewport = document.viewport.getDimensions();
		var main = $('main');

		var heightVal = viewport.height;
		// include body bottom margin into calculation (40)
		if ((main.getHeight() - 40) > viewport.height) {
			heightVal = main.getHeight() + 40;
		}
		// set modal transparent black overlay over entire page
		$('modalOverlay').setStyle({ height: heightVal + 'px' });

		// determine top position based on scroll offset and modal top offset
		var offset = main.cumulativeScrollOffset();
		var top = offset.top + modalTopOffset;

		// position modal content container
		$('modalOuterContainer').setStyle({ top: top + 'px' });


		// make sure following are initially hidden
		$('modalUtilNav').hide();
		$('modalScrollbarContainer').hide();

	}

}


// after modal content has loaded, determine if need to show vertical scrollbar base on browser window (viewport) dimensions
function setModalContentDimensions() {

	// only works if there's only one content section and not a wizard container
	if ($('modalContent') && $('modalScrollbarContainer')
			&& $('modalContent').select('.sectionContentContainer').size() == 1
			&& $('modalContent').select('.wizardContainer').size() == 0) {

		var viewport = document.viewport.getDimensions();
		// max height taking into consideration modal top and bottom offset spacing
		var maxModalHeight = viewport.height - modalTopOffset - modalBottomOffset - modalTopBottomHeight;

		var scrollAreaContainer = $('modalContent').select('.sectionContentContainer')[0];
		var scrollAreaContent = scrollAreaContainer.down('.sectionContent');

		// reset scroll area content position to zero/top
		scrollAreaContent.setStyle({ top: '0px' });

		// need to reset height to auto so that it can resize to content to see if becoming too large
		scrollAreaContainer.setStyle({ height: 'auto', overflow: 'visible' });

		// IE7 height discrepancy fix
		var specialBrowserOffset = 0;
		if (Prototype.Browser.IE && parseInt(navigator.userAgent.substring(navigator.userAgent.indexOf("MSIE") + 5)) == 7) {
			if ($('modalContent').down('h1')) {
				specialBrowserOffset = -15;
			} else {
				specialBrowserOffset = 15;
			}
		}
		// adjust max height with special browser offset
		maxModalHeight = maxModalHeight + specialBrowserOffset;

		// only need scrollbar if modal window content height is larger than the max height calculated
		if ($('modalContent').getHeight() > maxModalHeight) {

			// determine max content height based on current value, less the difference between the current modal height and the max height calculated
			var maxContentHeight = scrollAreaContainer.getHeight() - ($('modalContent').getHeight() - maxModalHeight);

			// prevent from hiding completely, min value
			if (maxContentHeight < 100) {
				maxContentHeight = 100;
			}

			// set scroll area container's height, and reset its overflow to hidden
			scrollAreaContainer.setStyle({ height: (maxContentHeight) + 'px', overflow: 'hidden' });

			// show vertical scrollbar and handle
			$('modalScrollbarContainer').show();
			$('modalScrollbarContainer').removeClassName('disabled');
			$('modalScrollbarContainer').down('.scrollBarHandle').show();

			// subtract 30px for height of h1
			var scrollbarHeight = $('modalContent').getHeight() - specialBrowserOffset;

			// if container has an h1, adjust height of scrollbar by 30px
			if ($('modalContent').down('h1')) {
				scrollbarHeight -= 30;
			}


			// adjust height scrollbar container and track's height
			$('modalScrollbarContainer').setStyle({ height: scrollbarHeight + 'px' });
			$('modalScrollbarTrack').setStyle({ height: scrollbarHeight - 5 + 'px' });

			// destroy slider object, if it already exists
			if (modalScrollbar) {
				modalScrollbar.dispose();
				modalScrollbar = null;
			}

			// setup slider object
			modalScrollbar = new Control.Slider($('modalScrollbarTrack').down('.scrollBarHandle'), $('modalScrollbarTrack'), {
				axis: 'vertical',
				// set range to match difference in scroll area content width versus its container height
				range: $R(0, (scrollAreaContent.getHeight() - scrollAreaContainer.getHeight())),
				// reset slider to zero/top
				sliderValue: 0,

				// need additional y offset to compensate for separate track bottom div
				alignY: -3,

				onSlide: function (value) {
					scrollAreaContent.setStyle({ top: -value + 'px' });
				},
				onChange: function (value) {
					scrollAreaContent.setStyle({ top: -value + 'px' });
				}
			});

		} else {
			// hide scrollbar
			$('modalScrollbarContainer').hide();
		}

	}

}


// open modal popup window
function openModal(url, specialModalClass) {

	if (url && $('main') && $('modalOverlay') && $('modalContent')) {

		if (specialModalClass) {
			// special modal classes, space separated for multiple
			//		'halfWidth' - sets popup window to be half standard width
			//		'hideClose' - hides close utility navigation item
			//		'hideHelp' - hides help utility navigation item

			// use split, in case there are multiple classes
			var specialModalClassArr = specialModalClass.split(' ');

			for (var i = 0; i < specialModalClassArr.length; i++) {
				$('modalOverlay').addClassName(specialModalClassArr[i]);
			}
		}


		// update content area
		new Ajax.Updater('modalContent', url, {
			method: 'get',
			evalScripts: true,
			onComplete: function () {
				// update any table borders once the content has loaded
				$('modalContent').select('.scrollAreaContainer').each(function (item) {
					setupScrollArea(item);
				});

				// see if modal content is too tall
				setModalContentDimensions();

				// show modal close 'X' button
				$('modalUtilNav').show();

				// update wizard page heights, if exist
				if ($('modalContent').down('.wizardPages')) {
					setWizardPageHeights($('modalContent').down('.wizardPages'));
				}
			}
		});

		// set intial modal overlay height and popup window top position
		setInitialModalDimensions();

		// show overlay
		$('modalOverlay').show();

	}

}

// close and hide all modal pieces
function closeModal() {

	if ($('modalOverlay')) {

		$('modalOverlay').hide();

		// clear all classes
		$('modalOverlay').classNames().each(function (tmpClassName) { $('modalOverlay').removeClassName(tmpClassName); });

		// clear content area
		$('modalContent').update();
	}

}


// wizard js before next page functions array variable
var wizardBeforeNextPageFunctionsArr = [];


// determines scrollable wizard page height based on tallest wizard page
function setWizardPageHeights(wizardPages) {

	if (wizardPages && wizardPages.down('.wizardPagesInner')) {

		var maxPageHeight = 0;

		// cycle through all wizard pages, finding the tallest value
		wizardPages.down('.wizardPagesInner').select('.wizardPage').each(function (item) { if (item.getDimensions().height > maxPageHeight) { maxPageHeight = item.getDimensions().height; } });

		// set both the the wizard page container, and inner container heights
		wizardPages.setStyle({ height: maxPageHeight + 'px' });
		wizardPages.down('.wizardPagesInner').setStyle({ height: maxPageHeight + 'px' });
	}

}


// update wizard tab on state
function updatelWizardTabs(wizardId, curPage) {

	var tabsId = wizardId + '_tabs';

	if ($(tabsId)) {

		if ($(tabsId).select('.wizardTab').size()) {

			var curTab = 0;
			// cycle through tabs, keeping track of tab number
			$(tabsId).select('.wizardTab').each(function (item) {

				// current page - add on state
				if (curTab == curPage) {
					item.removeClassName('disabled');
					item.addClassName('onTab');

					// next page - add disabled state
				} else if (curTab > curPage) {
					item.removeClassName('onTab');
					item.addClassName('disabled');

					// previous page - neither on nor disabled - default state
				} else {
					item.removeClassName('onTab');
					item.removeClassName('disabled');
				}

				curTab = curTab + 1;
			});

		}
	}
}


// update wizard buttons, between enabling/disabling previous/next buttons, and changing next to apply on final page
function updatelWizardButtons(wizardId, curPage, pageCount) {

	var buttonsId = wizardId + '_buttons';

	if ($(buttonsId)) {
		if ($(buttonsId).down('.wizardPrevious')) {
			if (curPage == 0) {
				// disable previous button
				$(buttonsId).down('.wizardPrevious').addClassName('stdButtonDisabled');
			} else {
				// enable previous button
				$(buttonsId).down('.wizardPrevious').removeClassName('stdButtonDisabled');
			}
		}

		if ($(buttonsId).down('.wizardNext') && $(buttonsId).down('.wizardNext').down('span')) {
			// last page - change button type to emphasized 2, and displayed text to 'apply'
			if ((curPage + 1) == pageCount) {
				$(buttonsId).down('.wizardNext').addClassName('stdButtonEmphasized2');
				$(buttonsId).down('.wizardNext').down('span').update(Translator.translate('generic.apply'));
				// not last page - change button type to default regular, and displayed text to 'next'
			} else {
				$(buttonsId).down('.wizardNext').removeClassName('stdButtonEmphasized2');
				$(buttonsId).down('.wizardNext').down('span').update(Translator.translate('generic.next'));
			}
		}

	}

}


// determine default scroll width based on first wizard page's width, if found
function getWizardScrollWidth(innerPagesId) {

	if ($(innerPagesId)) {
		var pageCount = $(innerPagesId).select('.wizardPage').size();

		if (pageCount > 0) {
			return $(innerPagesId).select('.wizardPage')[0].getDimensions().width;
		} else {
			// default scroll width
			return 705;
		}
	}

}

// determine current page based on current scroll area's offset and the default scroll width
function getCurrentPage(innerPagesId, scrollWidth) {

	if ($(innerPagesId)) {
		return -($(innerPagesId).positionedOffset()[0] / scrollWidth);
	}

}


// scroll wizard to specified page
function scrollToWizardPage(wizardId, pageNum) {

	var innerPagesId = wizardId + '_innerpages';

	if ($(innerPagesId)) {

		// determine total page count based on number of wizard pages
		var pageCount = $(innerPagesId).select('.wizardPage').size();

		// determine default scroll width
		var scrollWidth = getWizardScrollWidth(innerPagesId);

		// determine current page
		var curPage = getCurrentPage(innerPagesId, scrollWidth);
		// determine direction based on current page less page number passed in - negative is forward to the right, positive is backward to the left
		var direction = curPage - pageNum;

		// animate scroll
		new Effect.Move($(innerPagesId), { duration: paginationScrollSpeed,
			transition: Effect.Transitions.sinoidal,
			// move in direction, and number of pages
			x: scrollWidth * direction,
			y: 0,
			mode: 'relative',
			// after finish, update wizard tabs and buttons
			afterFinish: function () { updatelWizardTabs(wizardId, pageNum); updatelWizardButtons(wizardId, pageNum, pageCount); },
			// prevents another scroll event from ocurring until this one is completed
			queue: { scope: 'alertmove', limit: 1 }
		});
	}

}

// clicked on wizard
function wizardTabClicked(wizardId, tabNum) {

	var innerPagesId = wizardId + '_innerpages';
	var tabId = wizardId + '_tab_' + tabNum;

	if ($(innerPagesId) && $(tabId)) {
		// make sure tab not disabled
		if (!$(tabId).hasClassName('disabled')) {
			// scroll wizard to that tab's page
			scrollToWizardPage(wizardId, tabNum);
		}
	}

}


// clicked wizard next button, move to the that page
function wizardNextButtonClicked(wizardId) {

	var innerPagesId = wizardId + '_innerpages';
	var buttonsId = wizardId + '_buttons';

	if ($(innerPagesId) && $(buttonsId)) {

		// determine total page count
		var pageCount = $(innerPagesId).select('.wizardPage').size();

		// determine scroll default width
		var scrollWidth = getWizardScrollWidth(innerPagesId);

		// determine current and next page numbers
		var curPage = getCurrentPage(innerPagesId, scrollWidth);
		var pageNum = curPage + 1;

		// if last page, submit form
		if (pageNum == pageCount) {
			return true;

		} else {
			// run special pre page next function, if exists
			if (wizardBeforeNextPageFunctionsArr['deploy-os']) {
				wizardBeforeNextPageFunctionsArr['deploy-os'](curPage + 1);
			}

			// scroll to next page
			scrollToWizardPage(wizardId, pageNum);
		}
	}

	return false;

}

// clicked wizard previous button, move to the that page
function wizardPreviousButtonClicked(wizardId) {

	var innerPagesId = wizardId + '_innerpages';
	var buttonsId = wizardId + '_buttons';

	if ($(innerPagesId) && $(buttonsId)) {
		// make sure previous button is enabled
		if ($(buttonsId).down('.wizardPrevious') && !$(buttonsId).down('.wizardPrevious').hasClassName('stdButtonDisabled')) {
			var scrollWidth = getWizardScrollWidth(innerPagesId);

			// determine current and previous page numbers
			var curPage = getCurrentPage(innerPagesId, scrollWidth);
			var pageNum = curPage - 1;

			// scroll to previous page
			scrollToWizardPage(wizardId, pageNum);
		}
	}

	return false;

}

// clicked wizard cancel button, close modal window
function wizardCancelButtonClicked(wizardId) {

	var innerPagesId = wizardId + '_innerpages';

	if ($(innerPagesId) && $(innerPagesId).up('#modalContent')) {
		// close modal
		closeModal();
	}

	return false;

}

// sets the first elements height to the second, with an offset to fine tune
function fixContentHeights(id1, id2, offset) {

	if ($$(id1) && $$(id2)) {

		var div1 = $$(id1)[0];
		var div2 = $$(id2)[0];

		var height1 = div1.getHeight()
		var height2 = div2.getHeight();

		// set first elements height to that of the second plus offset
		div1.setStyle({ height: (height2 + offset) + 'px' });
	}

}


// syncs the two heights to the larger of the two, with an offset to fine tune
function syncContentHeights(id1, id2, offset) {

	if ($$(id1) && $$(id2)) {

		var div1 = $$(id1)[0];
		var div2 = $$(id2)[0];

		var height1 = div1.getHeight()
		var height2 = div2.getHeight();

		if ((height1 - offset) > height2) {
			div2.setStyle({ height: (height1 - offset) + 'px' });

		} else if ((height2 + offset) > height1) {
			div1.setStyle({ height: (height2 + offset) + 'px' });
		}
	}

}




// custom wizard functions

wizardBeforeNextPageFunctionsArr['deploy-os'] = function (curPage) {

	if ($('modalContent').down('form')) {
		var frm = $('modalContent').down('form');

		if (curPage == 1) {

			// show hide content blocks on second step based on operating system selected

			if (frm['operating_system']) {

				$('modalContent').down('#deploy_os_windows').hide();
				$('modalContent').down('#deploy_os_windows').select('input').each(function (item) { item.disable(); });
				$('modalContent').down('#deploy_os_linux').hide();
				$('modalContent').down('#deploy_os_linux').select('input').each(function (item) { item.disable(); });
				$('modalContent').down('#deploy_os_other').hide();
				$('modalContent').down('#deploy_os_other').select('input').each(function (item) { item.disable(); });


				var osVal = String($$('input:checked[type="radio"][name="operating_system"]').pluck('value'));

				$('modalContent').select('.os_name').each(function (item) { item.update(osVal); });

				if (osVal.indexOf('Windows') != -1) {
					$('modalContent').down('#deploy_os_windows').show();
					$('modalContent').down('#deploy_os_windows').select('input').each(function (item) { item.enable(); });

				} else if (osVal.indexOf('Linux') != -1) {
					$('modalContent').down('#deploy_os_linux').show();
					$('modalContent').down('#deploy_os_linux').select('input').each(function (item) { item.enable(); });

				} else {
					$('modalContent').down('#deploy_os_other').show();
					$('modalContent').down('#deploy_os_other').select('input').each(function (item) { item.enable(); });

				}
			}

		}
	}

	return true;

};




var scrollBar = {
	setupScrollBars: function () {
		// setup all scroll area containers on page
		$$('.scrollAreaContainer').each(function (item) { setupScrollArea(item); });
	}
};




function postPageLoadUpdate() {

	setTimeout(scrollBar.setupScrollBars, 100);
	setTimeout(SortableTable.find, 100);

}




//For the alerts control
var alerts = {
	create: function (alertsarray, tabpage) {
		var alertTemplate = new Template('<div class="alertItem"><div class="alertItemTitleDate"><div class="alertItemTitle">#{Title}</div><div class="alertItemDate">#{Date}</div></div><div class="alertItemText">#{Text}</div></div>');
		var alertTab;
		var alertItems = '';
		var alertContent;
		for (var i = 1; i <= alertsarray.length; i++) {
			//Create the Tabdiv and put 3 items at a time in it
			if (i % 3 == 1) {
				if (alertItems != '') this.write(alertTab, alertItems, tabpage);
				alertTab = document.createElement('div');
				alertTab.setAttribute('class', 'alertTabPage');
				alertItems = '';
			};
			alertItems += alertTemplate.evaluate(alertsarray[i - 1]);
		}
		//write the rest of the items
		this.write(alertTab, alertItems, tabpage);
	},
	write: function (alertTab, alertItems, tabpage) {
		alertTab.innerHTML = alertItems;
		$(tabpage).appendChild(alertTab);
	},
	writeheaders: function (data, containertitle, pagetext, tabtitle) {
		var paginationTemplate = new Template('#{PageText} <span>1</span> #{OfText} #{TotalPages}');
		$(containertitle).innerHTML = data.ContainerTitle;
		$(tabtitle).innerHTML = data.TabTitle;
		//calcuate the number of pages for the Template;
		var len = data.Alerts.length;
		data.TotalPages = len % 3 == 0 ? len / 3 : parseInt(len / 3) + 1;
		$(pagetext).innerHTML = paginationTemplate.evaluate(data);
	}
}
var configuration = {
	file: 'index.xml',
	initialize: function () {
		var file = window.configurationfile || this.file;
		var config = new Ajax.Request(file, { method: 'get', onSuccess: this.readConfig, onFailure: this.configError });
	},
	readConfig: function (request) {
		try {
			var response = request.responseXML;
			var config = response.childNodes;
			for (var i = 0; i < config.length; i++) {
				if (config[i].nodeName == "Nodes") {
					navigation.initailize(config[i].childNodes);
				}
			}
		}
		catch (err) {
			this.configError('Error');
		}
	},
	configError: function (request) {
		if (request != 'Error') {
			alert('Erroring reading configuration');
		} else {
			alert('Configuration file is empty or invalid');
		}
	}
};

