/**
 * Session storage wrapper
 * @Author      Circumerro Media
 *              Borrows concepts from jStorage
 * @DEPENDENCY  Prototype
 */
(function($){

    // does browser support sessionStorage?
    var _supported = (function(){
       try {
           return 'sessionStorage' in window && window['sessionStorage'] !== null;
       }catch(e){
           return false;
       }
    })();

    $.sessionizer = (function(supportedBrowser){

        function _checkKey(key){
            if(supportedBrowser){
                if(!key || (typeof key != "string" && typeof key != "number")){
                    throw new TypeError('Key name must be string or numeric');
                }
                return true;
            }else{
                return false;
            }
        }

        return {
            set:function(k,v){
                if(_checkKey(k)){
                    sessionStorage[k] = v;
                    return true;
                }
                return false;
            },
            get:function(k){
                if(_checkKey(k)){
                    return sessionStorage[k] || false;
                }
                return false;
            },
            deleteKey:function(k){
                if(_checkKey(k)){
                    if(k in sessionStorage){
                        delete sessionStorage[k];
                        return true;
                    }
                }
                return false;
            },
            isAvailable:function(){
                return supportedBrowser;
            }
        }

    })(_supported);

})(window.$)